﻿local GCD = LibStub("AceAddon-3.0"):NewAddon("GCD", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("GCD")
local ACD = LibStub("AceConfigDialog-3.0")

local GetTime = GetTime

local GCDBar
local starttime, duration, spellid, warned, caststarttime, castendtime, castduration, castinterrupted, castlatency, castsent, iscasting, isgcd

local frame1, frame2, frame3, frame4 = {},{},{},{}
local frameBuf = {}

local sin, cos, rad, ceil, floor = math.sin, math.cos, math.rad, math.ceil, math.floor

local BOOKTYPE_SPELL = BOOKTYPE_SPELL

function GCD:InitializeOptions()
	self.options = {
		name = "GCD",
		type = "group",
		args = {
			spell = {
				name = L["Spell"],
				desc = L["Set spell to track cooldowns"],
				type = "input",
				get = function(info) return self.db.char.spell end,
				set = function(info,val)
							self.db.char.spell = strlower(val)
							self:SPELLS_CHANGED()
						end,
				order = 100
			},
			info = {
				name = L["All specified colors are vertex colors!"],
				type = "description",
				order = 200
			},
			spark = {
				name = L["Spark"],
				type = "group",
				args = {
					texture = {
						name = L["Texture"],
						type = "input",
						get = function(info) return self.db.profile.spark.texture end,
						set = function(info, val)
									self.db.profile.spark.texture = val
									self:ApplySettings()
								end,
						order = 100
					},
					color = {
						name = L["Color"],
						type = "color",
						get = function(info) return self.db.profile.spark.color.r, self.db.profile.spark.color.g, self.db.profile.spark.color.b, self.db.profile.spark.color.a end,
						set = function(info, r, g, b, a)
									self.db.profile.spark.color = {r=r, g=g, b=b, a=a}
									self:ApplySettings()
								end,
						hasAlpha = true,
						order = 200
					},
				}
			},
			ring = {
				name = L["Ring"],
				type = "group",
				args = {
					texture = {
						name = L["Texture"],
						type = "input",
						get = function(info) return self.db.profile.ring.texture end,
						set = function(info, val)
									self.db.profile.ring.texture = val
									self:ApplySettings()
								end,
						order = 100
					},
					color = {
						name = L["Color"],
						type = "color",
						get = function(info) return self.db.profile.ring.color.r, self.db.profile.ring.color.g, self.db.profile.ring.color.b, self.db.profile.ring.color.a end,
						set = function(info, r, g, b, a)
									self.db.profile.ring.color = {r=r, g=g, b=b, a=a}
									self:ApplySettings()
								end,
						hasAlpha = true,
						order = 200
					},
					rotate = {
					name = L["Rotate"],
					type = "toggle",
					get = function(info) return self.db.profile.ring.rotate end,
					set = function(info, val) self.db.profile.ring.rotate = val end,
					order = 300
					}
				}
			},
			casttime = {
				name = L["Casttime"],
				type = "group",
				args = {
					enabled = {
						name = L["Enabled"],
						type = "toggle",
						get = function(info) return self.db.char.casttime end,
						set = function(info,val)
								self.db.char.casttime = val
								if val then
									self:RegisterEvent("UNIT_SPELLCAST_SENT")
									self:RegisterEvent("UNIT_SPELLCAST_START")
									self:RegisterEvent("UNIT_SPELLCAST_STOP")
									self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
									self:RegisterEvent("UNIT_SPELLCAST_FAILED")
									self:RegisterEvent("UNIT_SPELLCAST_DELAYED")
									self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
									self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
									self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
								end
							end,
						order = 100
					},
					texture = {
						name = L["Texture"],
						type = "input",
						get = function(info) return self.db.profile.casttime.texture end,
						set = function(info, val)
									self.db.profile.casttime.texture = val
									self:ApplySettings()
								end,
						order = 100
					},
					castcolor = {
						name = L["Cast color"],
						type = "color",
						get = function(info) return self.db.profile.casttime.castcolor.r, self.db.profile.casttime.castcolor.g, self.db.profile.casttime.castcolor.b, self.db.profile.casttime.castcolor.a end,
						set = function(info, r, g, b, a)
									self.db.profile.casttime.castcolor = {r=r, g=g, b=b, a=a}
									self:ApplySettings()
								end,
						hasAlpha = true,
						order = 200
					},
					latencycolor = {
						name = L["Latency color"],
						type = "color",
						get = function(info) return self.db.profile.casttime.latencycolor.r, self.db.profile.casttime.latencycolor.g, self.db.profile.casttime.latencycolor.b, self.db.profile.casttime.latencycolor.a end,
						set = function(info, r, g, b, a)
									self.db.profile.casttime.latencycolor = {r=r, g=g, b=b, a=a}
									self:ApplySettings()
								end,
						hasAlpha = true,
						order = 300
					},
					interruptcolor = {
						name = L["Interrupt color"],
						type = "color",
						get = function(info) return self.db.profile.casttime.interruptcolor.r, self.db.profile.casttime.interruptcolor.g, self.db.profile.casttime.interruptcolor.b, self.db.profile.casttime.interruptcolor.a end,
						set = function(info, r, g, b, a)
									self.db.profile.casttime.interruptcolor = {r=r, g=g, b=b, a=a}
									self:ApplySettings()
								end,
						hasAlpha = true,
						order = 300
					},
					hideblizzard = {
						name = L["Hide default castbar"],
						type = "toggle",
						get = function(info) return self.db.char.hidecastbar end,
						set = function(info,val)
								self.db.char.hidecastbar = val
								if val then
									CastingBarFrame:UnregisterAllEvents()
									CastingBarFrame:Hide()
								else
									CastingBarFrame:UnregisterAllEvents()
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START")
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP")
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED")
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED")
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
								end
							end,
						order = 100
					},
				}
			},
			spellcds = {
				name = L["Spellcds"],
				type = "group",
				args = {
					enabled = {
						name = L["Enabled"],
						type = "toggle",
						get = function(info) return self.db.char.spellcds end,
						set = function(info,val)
								self:ApplySettings()
								self.db.char.spellcds = val
								if val then
									self:ACTIONBAR_UPDATE_COOLDOWN()
								else
									if frame1.frame then frame1.frame:Hide() end
									if frame2.frame then frame2.frame:Hide() end
									if frame3.frame then frame3.frame:Hide() end
									if frame4.frame then frame4.frame:Hide() end
								end
							end,
						order = 100
					},
				}
			}
		}
	}
end

local function OnUpdate(self, elapsed)
	if not iscasting and not isgcd then
		GCDBar:Hide()
	end
	local x, y = GetCursorPosition(UIParent);
	self:SetPoint('CENTER', UIParent, 'BOTTOMLEFT', (x+self.xOff)/self:GetEffectiveScale(), (y+self.yOff)/self:GetEffectiveScale())
		
	if self.spark then
		if GCD.db.profile.ring.rotate then
			self.texture.timer = self.texture.timer + elapsed;
			if ( self.texture.timer > 0.02 ) then
				self.texture.hAngle = self.texture.hAngle + 0.5;
				local s = sin(rad(self.texture.hAngle));
				local c = cos(rad(self.texture.hAngle));
				self.texture:SetTexCoord(	0.5-s, 0.5+c,
													0.5+c, 0.5+s,
													0.5-c, 0.5-s,
													0.5+s, 0.5-c);
				self.texture.timer = 0;
			end
		end
		if isgcd then
			local gcdperc = (GetTime() - self.starttime) / self.duration
			gcdperc = gcdperc < 1 and gcdperc or 1			
			local angle = rad(90-gcdperc*360)
			local x, y = cos(angle), sin(angle)
			self.spark:SetPoint('CENTER', self, 'CENTER',x*25,y*25)
			self.spark:Show()
			if gcdperc >= 1 then
				isgcd = false
				self.spark:Hide()
			end
		else
			self.spark:Hide()
		end
		if iscasting and GCD.db.char.casttime then
			local castperc = (1000 * GetTime() - caststarttime) / castduration
			castperc = castperc < 1 and castperc or 1
			if castperc > 1-castlatency then
				self.castspark:SetVertexColor(GCD.db.profile.casttime.latencycolor.r, GCD.db.profile.casttime.latencycolor.g, GCD.db.profile.casttime.latencycolor.b, GCD.db.profile.casttime.latencycolor.a) -- 1,0,0,1
			elseif castinterrupted then
				self.castspark:SetVertexColor(GCD.db.profile.casttime.interruptcolor.r, GCD.db.profile.casttime.interruptcolor.g, GCD.db.profile.casttime.interruptcolor.b, GCD.db.profile.casttime.interruptcolor.a) -- 1,1,0,1
			else
				self.castspark:SetVertexColor(GCD.db.profile.casttime.castcolor.r, GCD.db.profile.casttime.castcolor.g, GCD.db.profile.casttime.castcolor.b, GCD.db.profile.casttime.castcolor.a) -- 0,1,0,1
			end
			local angle = rad(90-castperc*360)
			local x, y = cos(angle), sin(angle)
			self.castspark:SetPoint('CENTER', self, 'CENTER',x*30,y*30)
			self.castspark:Show()
			if castperc >= 1 then
				iscasting = false
				self.castspark:Hide()
			end
		else
			self.castspark:Hide()
		end
	elseif self.spell and GCD.db.char.spellcds then
		local perc = (GetTime() - self.starttime) / self.duration
		self.texture:SetVertexColor(1.0,perc,perc)
		local dur = floor((self.duration - (GetTime() - self.starttime))*10)/10
		if dur > 99 then
			dur = '>>'
		elseif dur > 10 then
			dur = ceil(dur)
		end
		self.cdtext:SetText(dur)
		if perc >= 1 then
			self:Hide()
		end
	end
end

local function OnHide(self)
	self:SetScript('OnUpdate', nil)
end

local function OnShow(self)
	self:SetScript('OnUpdate', OnUpdate)
end

local defaults = {
	char = {
		spell = '',
		spellcds = false,
		casttime = false,
		hidecastbar = false,
	},
	profile = {
		spark = {
			color = {r=1, g=1, b=1, a=1},
			texture = "Spells\\AURA_01"
		},
		ring = {
			color = {r=0, g=1, b=0, a=0.5},
			texture = "Spells\\AURARUNE8",
			rotate = true
		},
		casttime = {
			castcolor = {r=0, g=1, b=0, a=1},
			latencycolor = {r=1, g=0, b=0, a=1},
			interruptcolor = {r=1, g=1, b=0, a=1},
			texture = "Spells\\AURA_01",
		},
	}
}

local function GetSpellID(spellname)
	local spellid = nil
	for tab = 1, 4 do
		local _, _, offset, numSpells = GetSpellTabInfo(tab)
		for i = (1+offset), (offset+numSpells) do
			local spell = GetSpellName(i, BOOKTYPE_SPELL)
			if strlower(spell) == strlower(spellname) then
				spellid = i
				break
			end
		end
	end
	return spellid;
end

function GCD:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("GCDDB", defaults)
	self:InitializeOptions()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("GCD", self.options)
	self:RegisterChatCommand("gcd",self.OpenConfig)
end

function GCD:ApplySettings()
	if not GCDBar then
		GCDBar = CreateFrame('Frame', 'GCDBar', UIParent)
	end
	GCDBar:Hide()
	GCDBar:ClearAllPoints()
	GCDBar:SetFrameStrata('HIGH')
	GCDBar:SetScript('OnShow', OnShow)
	GCDBar:SetScript('OnHide', OnHide)
	GCDBar:SetWidth(64)
	GCDBar:SetHeight(64)
	GCDBar.xOff = 10
	GCDBar.yOff = -10
	
	if not GCDBar.texture then
		GCDBar.texture = GCDBar:CreateTexture(nil, 'ARTWORK')
		GCDBar.texture.timer = 0;
		GCDBar.texture.hAngle = 0;
	end
	GCDBar.texture:SetTexture(self.db.profile.ring.texture) -- Spells\\AURARUNE8
	GCDBar.texture:SetVertexColor(self.db.profile.ring.color.r,self.db.profile.ring.color.g,self.db.profile.ring.color.b,self.db.profile.ring.color.a) -- 0,1,0,0.5
	GCDBar.texture:SetBlendMode('ADD')
	GCDBar.texture:SetWidth(80)
	GCDBar.texture:SetHeight(80)
	GCDBar.texture:SetPoint('CENTER', GCDBar, 'CENTER')
	local s = sin(rad(GCDBar.texture.hAngle));
	local c = cos(rad(GCDBar.texture.hAngle));
	GCDBar.texture:SetTexCoord(0.5-s, 0.5+c,
										0.5+c, 0.5+s,
										0.5-c, 0.5-s,
										0.5+s, 0.5-c);
	
	if not GCDBar.spark then
		GCDBar.spark = GCDBar:CreateTexture(nil, 'DIALOG')
	end
	GCDBar.spark:SetTexture(self.db.profile.spark.texture) -- Spells\\AURA_01
	GCDBar.spark:SetBlendMode('ADD')
	GCDBar.spark:SetVertexColor(self.db.profile.spark.color.r,self.db.profile.spark.color.g,self.db.profile.spark.color.b,self.db.profile.spark.color.a) -- 1,1,1
	GCDBar.spark:SetWidth(30)
	GCDBar.spark:SetHeight(30)
	
	if not GCDBar.castspark then
		GCDBar.castspark = GCDBar:CreateTexture(nil, 'DIALOG')
	end
	GCDBar.castspark:SetTexture(self.db.profile.casttime.texture) -- Spells\\AURA_01
	GCDBar.castspark:SetVertexColor(self.db.profile.casttime.castcolor.r, self.db.profile.casttime.castcolor.g, self.db.profile.casttime.castcolor.b, self.db.profile.casttime.castcolor.a) -- 0,1,0,1
	GCDBar.castspark:SetBlendMode('ADD')
	GCDBar.castspark:SetWidth(30)
	GCDBar.castspark:SetHeight(30)
	
	if not frame1.frame and frame1.spell then
		local frame
		if #frameBuf > 0 then
			frame = tremove(frameBuf)
		else
			frame = CreateFrame('Frame', 'GCDFrame1', UIParent)
		end
		frame:ClearAllPoints()
		frame:SetFrameStrata('HIGH')
		frame:SetWidth(1)
		frame:SetHeight(1)
		frame:SetScript('OnShow', OnShow)
		frame:SetScript('OnHide', OnHide)
		frame:Hide()
		frame.xOff = -20
		frame.yOff = 20
		frame.spell = true
		
		frame.texture = frame.texture or frame:CreateTexture(nil, 'ARTWORK')
		frame.texture:SetTexture(frame1.icon)
		frame.texture:SetVertexColor(1,0,0)
		frame.texture:SetWidth(15*frame:GetEffectiveScale())
		frame.texture:SetHeight(15*frame:GetEffectiveScale())
		frame.texture:SetPoint('CENTER', frame, 'CENTER')
		
		frame.cdtext = frame.cdtext or frame:CreateFontString()
		frame.cdtext:SetFont("Fonts\\FRIZQT__.TTF", 11, "OUTLINE, MONOCHROME")
		frame.cdtext:SetPoint('CENTER', frame, 'CENTER', 0, 1)
		frame1.frame = frame
	elseif frame1.frame and not frame1.spell then
		local frame = frame1.frame
		frame1.frame = nil
		frame:Hide()
		tinsert(frameBuf, frame)
	elseif frame1.frame then
		frame1.frame.texture:SetTexture(frame1.icon)
	end
	if not frame2.frame and frame2.spell then
		local frame
		if #frameBuf > 0 then
			frame = tremove(frameBuf)
		else
			frame = CreateFrame('Frame', 'GCDFrame2', UIParent)
		end
		frame:ClearAllPoints()
		frame:SetFrameStrata('HIGH')
		frame:SetWidth(1)
		frame:SetHeight(1)
		frame:SetScript('OnShow', OnShow)
		frame:SetScript('OnHide', OnHide)
		frame:Hide()
		frame.xOff = 40
		frame.yOff = 20
		frame.spell = true
		
		frame.texture = frame.texture or frame:CreateTexture(nil, 'ARTWORK')
		frame.texture:SetTexture(frame2.icon)
		frame.texture:SetVertexColor(1,0,0)
		frame.texture:SetWidth(15*frame:GetEffectiveScale())
		frame.texture:SetHeight(15*frame:GetEffectiveScale())
		frame.texture:SetPoint('CENTER', frame, 'CENTER')
		
		frame.cdtext = frame.cdtext or frame:CreateFontString()
		frame.cdtext:SetFont("Fonts\\FRIZQT__.TTF", 11, "OUTLINE, MONOCHROME")
		frame.cdtext:SetPoint('CENTER', frame, 'CENTER', 0, 1)
		frame2.frame = frame
	elseif frame2.frame and not frame2.spell then
		local frame = frame2.frame
		frame2.frame = nil
		frame:Hide()
		tinsert(frameBuf, frame)
	elseif frame2.frame then
		frame2.frame.texture:SetTexture(frame2.icon)
	end
	if not frame3.frame and frame3.spell then
		local frame
		if #frameBuf > 0 then
			frame = tremove(frameBuf)
		else
			frame = CreateFrame('Frame', 'GCDFrame3', UIParent)
		end
		frame:ClearAllPoints()
		frame:SetFrameStrata('HIGH')
		frame:SetWidth(1)
		frame:SetHeight(1)
		frame:SetScript('OnShow', OnShow)
		frame:SetScript('OnHide', OnHide)
		frame:Hide()
		frame.xOff = -20
		frame.yOff = -40
		frame.spell = true
		
		frame.texture = frame.texture or frame:CreateTexture(nil, 'ARTWORK')
		frame.texture:SetTexture(frame3.icon)
		frame.texture:SetVertexColor(1,0,0)
		frame.texture:SetWidth(15*frame:GetEffectiveScale())
		frame.texture:SetHeight(15*frame:GetEffectiveScale())
		frame.texture:SetPoint('CENTER', frame, 'CENTER')
		
		frame.cdtext = frame.cdtext or frame:CreateFontString()
		frame.cdtext:SetFont("Fonts\\FRIZQT__.TTF", 11, "OUTLINE, MONOCHROME")
		frame.cdtext:SetPoint('CENTER', frame, 'CENTER', 0, 1)
		frame3.frame = frame
	elseif frame3.frame and not frame3.spell then
		local frame = frame3.frame
		frame3.frame = nil
		frame:Hide()
		tinsert(frameBuf, frame)
	elseif frame3.frame then
		frame3.frame.texture:SetTexture(frame3.icon)
	end
	if not frame4.frame and frame4.spell then
		local frame
		if #frameBuf > 0 then
			frame = tremove(frameBuf)
		else
			frame = CreateFrame('Frame', 'GCDFrame4', UIParent)
		end
		frame:ClearAllPoints()
		frame:SetFrameStrata('HIGH')
		frame:SetWidth(1)
		frame:SetHeight(1)
		frame:SetScript('OnShow', OnShow)
		frame:SetScript('OnHide', OnHide)
		frame:Hide()
		frame.xOff = 40
		frame.yOff = -40
		frame.spell = true
		
		frame.texture = frame.texture or frame:CreateTexture(nil, 'ARTWORK')
		frame.texture:SetTexture(frame4.icon)
		frame.texture:SetVertexColor(1,0,0)
		frame.texture:SetWidth(15*frame:GetEffectiveScale())
		frame.texture:SetHeight(15*frame:GetEffectiveScale())
		frame.texture:SetPoint('CENTER', frame, 'CENTER')
		
		frame.cdtext = frame.cdtext or frame:CreateFontString()
		frame.cdtext:SetFont("Fonts\\FRIZQT__.TTF", 11, "OUTLINE, MONOCHROME")
		frame.cdtext:SetPoint('CENTER', frame, 'CENTER', 0, 1)
		frame4.frame = frame
	elseif frame4.frame and not frame4.spell then
		local frame = frame4.frame
		frame4.frame = nil
		frame:Hide()
		tinsert(frameBuf, frame)
	elseif frame4.frame then
		frame4.frame.texture:SetTexture(frame4.icon)
	end
end

function GCD:OpenConfig()
	ACD:Open("GCD")
end

function GCD:OnEnable()
	if self.db.char.casttime then
		self:RegisterEvent("UNIT_SPELLCAST_SENT")
		self:RegisterEvent("UNIT_SPELLCAST_START")
		self:RegisterEvent("UNIT_SPELLCAST_STOP")
		self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
		self:RegisterEvent("UNIT_SPELLCAST_FAILED")
		self:RegisterEvent("UNIT_SPELLCAST_DELAYED")
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
	end
	if self.db.char.hidecastbar then
		CastingBarFrame:UnregisterAllEvents()
		CastingBarFrame:Hide()
	end
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")
	self:RegisterEvent("SPELLS_CHANGED")
	self:SPELLS_CHANGED()
end

function GCD:OnDisable()
	GCDBar:Hide()
	self:UnregisterAllEvents()
	if self.db.char.hidecastbar then
		CastingBarFrame:UnregisterAllEvents()
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
	end
end

function GCD:ACTIONBAR_UPDATE_COOLDOWN()
	if spellid then
		local start, dur = GetSpellCooldown(spellid, BOOKTYPE_SPELL)
		if dur > 0 and dur <= 2 then
			isgcd = true
			GCDBar.starttime = start
			GCDBar.duration = dur
			GCDBar:Show()
		end
		if self.db.char.spellcds then
			local _, odur = GetSpellCooldown(spellid, BOOKTYPE_SPELL)
			if frame1.spell then
				local start, dur = GetSpellCooldown(frame1.spell, BOOKTYPE_SPELL)
				if dur > odur then
					frame1.frame.starttime = start
					frame1.frame.duration = dur
					frame1.frame:Show()
				end
			end
			if frame2.spell then
				local start, dur = GetSpellCooldown(frame2.spell, BOOKTYPE_SPELL)
				if dur > odur then
					frame2.frame.starttime = start
					frame2.frame.duration = dur
					frame2.frame:Show()
				end
			end
			if frame3.spell then
				local start, dur = GetSpellCooldown(frame3.spell, BOOKTYPE_SPELL)
				if dur > odur then
					frame3.frame.starttime = start
					frame3.frame.duration = dur
					frame3.frame:Show()
				end
			end
			if frame4.spell then
				local start, dur = GetSpellCooldown(frame4.spell, BOOKTYPE_SPELL)
				if dur > odur then
					frame4.frame.starttime = start
					frame4.frame.duration = dur
					frame4.frame:Show()
				end
			end
		end
	end
end

function GCD:UNIT_SPELLCAST_SENT(_, unit)
	if unit ~= 'player' then return end
	castsent = GetTime()*1000
end

function GCD:UNIT_SPELLCAST_START(_, unit)
	if unit ~= 'player' then return end
	iscasting = true
	castinterrupted = false
	_, _, _, icon, caststarttime, castendtime = UnitCastingInfo(unit)
	local sendlag= (castsent and castsent > 0) and GetTime()*1000-castsent or 0
	castduration=castendtime-caststarttime
	sendlag = sendlag > castduration and castduration or sendlag
	castlatency = sendlag/castduration
	GCDBar:Show()
end

function GCD:UNIT_SPELLCAST_STOP(_, unit)
	if unit ~= 'player' then return end
  iscasting = false
end

function GCD:UNIT_SPELLCAST_FAILED(event, unit)
	if unit ~= 'player' then return end
	iscasting = false
end

function GCD:UNIT_SPELLCAST_INTERRUPTED(event, unit)
	if unit ~= 'player' then return end
	iscasting = false
end

function GCD:UNIT_SPELLCAST_DELAYED(event, unit)
	if unit ~= 'player' then return end
	castinterrupted = true
	_, _, _, _, caststarttime, castendtime = UnitCastingInfo(unit)
end

function GCD:UNIT_SPELLCAST_CHANNEL_START(event,unit)
	if unit ~= 'player' then return end
	iscasting = true
	_, _, _, _, caststarttime, castendtime = UnitChannelInfo(unit)
	local sendlag= (castsent and castsent > 0) and GetTime()*1000-castsent or 0
	castduration=castendtime-caststarttime
	sendlag = sendlag > castduration and castduration or sendlag
	castlatency = sendlag/castduration
	GCDBar:Show()
end

function GCD:UNIT_SPELLCAST_CHANNEL_STOP(event,unit)
	if unit ~= 'player' then return end
	iscasting = false
end

function GCD:UNIT_SPELLCAST_CHANNEL_UPDATE(event,unit)
	if unit ~= 'player' then return end
	castinterrupted = true
	_, _, _, _, caststarttime, castendtime = UnitChannelInfo(unit)
end

function GCD:SPELLS_CHANGED()
	if not self.db.char.spell then
		if not warned then
			self:Print(L["No spell set! Set one with: /gcd spell <spellname>"])
			warned = true
		end
	else
		spellid = GetSpellID(self.db.char.spell)
		if not spellid then
			self:Print(L["Invalid spell set! Set new with: /gcd spell <spellname>"])
			self.db.char.spell = nil
			warned = true
		end
	end
	local spell, icon
	local _, class = UnitClass("player")
	if class == "DRUID" then
		spell, _, icon = GetSpellInfo(53251) -- Wild Growth
		frame1.spell, frame1.icon = GetSpellID(spell), icon
		spell, _, icon = GetSpellInfo(18562) -- Swiftmend
		frame2.spell, frame2.icon = GetSpellID(spell), icon
		spell, _, icon = GetSpellInfo(17116) -- Nature's Swiftness
		frame3.spell, frame3.icon = GetSpellID(spell), icon
	elseif class == "PALADIN" then
		spell, _, icon = GetSpellInfo(48825) -- Holy Shock
		frame1.spell, frame1.icon = GetSpellID(spell), icon
	elseif class == "PRIEST" then
		spell, _, icon = GetSpellInfo(53007) -- Penance
		frame1.spell, frame1.icon = GetSpellID(spell), icon
		spell, _, icon = GetSpellInfo(48113) -- Prayer of Mending
		frame2.spell, frame2.icon = GetSpellID(spell), icon
		spell, _, icon = GetSpellInfo(48089) -- Circle of Healing
		frame3.spell, frame3.icon = GetSpellID(spell), icon
	end
	self:ApplySettings()
end
